<?

error_reporting(E_ALL);
ini_set('display_errors', '1');

/**
 * Constants
 */
define('CDN_BASE', 'http://archive.gersteinlab.org/static');
 
define('PAPERS_URL', 'http://papers.gersteinlab.org/papers/');
define('SRC_PATH', '/Users/dzc/Work/aptana3-workspace/GersteinLab/imageupload/images');
define('OLD_PATH', '/Users/dzc/Work/aptana3-workspace/GersteinLab/imageupload/old');
define('MAX_IMG_SIZE', 50000000);
define('IMG_HEIGHT', 320);
define('IMG_WIDTH', 650);
define('LOG_FILE', 'summary.txt');

$accepted_exts = array('jpg', 'jpeg', 'png', 'gif');

/**
 * Globals
 */
$fatal = array();
$errors = array(
  'uploadFile'    => NULL,
  'fileSubgroup'    => NULL,
  'fileSubmitter'    => NULL,
  'fileDescription'  => NULL,
);
$values = array(
  'uploadFile'    => NULL,
  'fileSubgroup'    => NULL,
  'fileSubmitter'    => NULL,
  'fileDescription'  => NULL
);
$blurb = '';


function http_response($url, $status = null, $wait = 3) { 
  $time = microtime(true); 
  $expire = $time + $wait; 

  $ch = curl_init(); 
  curl_setopt($ch, CURLOPT_URL, $url); 
  curl_setopt($ch, CURLOPT_HEADER, TRUE); 
  curl_setopt($ch, CURLOPT_NOBODY, TRUE); // remove body 
  curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE); 
  $head = curl_exec($ch); 
  $httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE); 
  curl_close($ch); 
  
  if (!$head) {
    return FALSE; 
  } 
  
  if ($status === null) 
  {
    if ($httpCode < 400) { 
      return TRUE; 
    } else { 
      return FALSE; 
    } 
  } else if ($status == $httpCode) { 
      return TRUE; 
  } 
} 

function strip_ext($filename) {
  $ext = strrchr($filename, '.');  
  if ($ext !== false) {  
    $filename = substr($filename, 0, -strlen($ext));  
  }  
  return $filename;
}

function print_error($field) {
  global $errors;
  
  if ($errors[$field] != NULL) {
    echo '<span style="color: red;">'.$errors[$field].'</span>';
  }
}

function strip_paper_id($file_name) {
  $stripped = strip_ext($file_name);
  $delim_pos = strpos($file_name, '__');
  return substr($file_name, 0, $delim_pos);
}

function validate_form($post, $files, $file_name)
{
  global $errors, $accepted_exts;
  
  $success = TRUE;
  $errors['uploadFile'] = '';
  
  // Check uploaded file
  if ($files['uploadFile']['tmp_name'] == '') {
    $errors['uploadFile'] = 'No file specified';
    $success = FALSE;
  } else {
    $parts = pathinfo($files['uploadFile']['name']);
    if ( ! in_array($parts['extension'], $accepted_exts)) {
      $errors['uploadFile'] = 'Invalid image extension ' . $parts['extension'] . '.';
      $success = FALSE;
    }
    
    if (file_exists($file_name)) {
      $errors['uploadFile'] .= " Image with file name " . $file_name . " already exists.";
      $success = FALSE;
    }
    
    $size = filesize($files['uploadFile']['tmp_name']);
    if ( ! $size || $size > MAX_IMG_SIZE) {
      $errors['uploadFile'] .= ' Image exceeds allowed file size.';
      $success = FALSE;
    }
    
    if ( ! preg_match('/[a-zA-Z0-9]+__[0-9a-zA-Z_.]+/', $file_name)) {
      $errors['uploadFile'] .= ' Image file name format invalid.';
      $success = FALSE;
    }
    
    $imagesize = getimagesize($files['uploadFile']['tmp_name']);
    if (!$imagesize || $imagesize[0] != IMG_WIDTH || $imagesize[1] != IMG_HEIGHT) {
      $errors['uploadFile'] .= ' Images must be ' . IMG_WIDTH . 'x' . IMG_HEIGHT
                    .'. Your image is ' . $imagesize[0] . 'x' . $imagesize[1] . '.';
      $success = FALSE;
    }
    
    $paper_url = build_url($files['uploadFile']['name']);
    if (!http_response($paper_url)) {
      $errors['uploadFile'] .= ' Invalid paper ID.';
      $success = FALSE;
    }
  }
  
  // Check other form fields
  if ($post['fileSubgroup'] == '') {
    $errors['fileSubgroup'] = 'No subgroup specified';
    $success = FALSE;
  }
  if ($post['fileSubmitter'] == '') {
    $errors['fileSubmitter'] = 'No submitter specified';
    $success = FALSE;
  }
  
  return $success;
}

function build_url($file_name) {
  $url = PAPERS_URL . strip_paper_id($file_name);
  return $url;
}

function sanitize_file_name($file_name) {
  $file_name = stripslashes($file_name);
  $file_name = str_replace("'" , "", $file_name);
  $file_name = str_replace(".php", "", $file_name);
  $file_name = str_replace(".pl", "", $file_name);
  $file_name = str_replace(".cgi", "", $file_name);
  $file_name = str_replace(".py", "", $file_name);
  return $file_name;
}

function handle_upload() {
  global $errors, $fatal, $blurb;
  
  if (($summary_file = fopen("./summary.txt", 'a')) == FALSE) {
    array_push($fatal, "Cannot open summary log");
    return FALSE;
  }
  $file_name = sanitize_file_name($_FILES['uploadFile']['name']);
  if (($check = validate_form($_POST, $_FILES, $file_name)) == FALSE) {
    return FALSE;
  }
  
  $paper_url = build_url($file_name);
  if (($copy = copy($_FILES['uploadFile']['tmp_name'], './images/'.$file_name)) == FALSE) {
    array_push($fatal, "Cannot upload file ".$file_name);
    return FALSE;
  }
  
  $blurb = 'Your file, <a href="./images/'.$file_name.'">'
      .$_FILES['uploadFile']['name'].'</a> '
        .'has been uploaded!';
  $file_entry = $file_name.','.date("Ymd").','
         .$_POST['fileSubmitter'].','
         .$_POST['fileSubgroup'].','
         .$_POST['fileDescription'];
             
  fwrite($summary_file, $file_entry);
  fclose($summary_file);

  return TRUE;
}

function rebuild_homepage()
{
  $last = system('./syncsplash.py', $retval);
  
  echo $retval . "\n";
  echo $last;
  
  if ($last === FALSE || $retval != 0)
    return FALSE;
  else
    return TRUE;
}

/**
 * View section
 */
?>


<!doctype html>
<html lang="en" class="no-js">
<head>
  <meta charset="utf-8">

  <!-- www.phpied.com/conditional-comments-block-downloads/ -->
  <!--[if IE]><![endif]-->

  <!-- Always force latest IE rendering engine (even in intranet) & Chrome Frame 
     Remove this if you use the .htaccess -->
  <meta http-equiv="X-UA-Compatible" content="IE=edge,chrome=1">

  <title>Gerstein Lab</title>
  <meta name="description" content="">
  <meta name="author" content="">

  <!--  Mobile Viewport Fix
      j.mp/mobileviewport & davidbcalhoun.com/2010/viewport-metatag 
  device-width : Occupy full width of the screen in its current orientation
  initial-scale = 1.0 retains dimensions instead of zooming out if page height > device height
  maximum-scale = 1.0 retains dimensions instead of zooming in if page width < device width
  -->
  <meta name="viewport" content="width=device-width; initial-scale=1.0; maximum-scale=1.0;">

  <!-- Place favicon.ico and apple-touch-icon.png in the root of your domain and delete these references -->
  <link rel="shortcut icon" href="favicon.ico">
  <link rel="apple-touch-icon" href="apple-touch-icon.png">

  <!-- CSS : implied media="all" -->
  <link rel="stylesheet" href="<? echo CDN_BASE; ?>/css/style.css?v=1">

  <!-- All JavaScript at the bottom, except for Modernizr which enables HTML5 elements & feature detects -->
  <script language="JavaScript" src="<? echo CDN_BASE; ?>media/js/modernizr-1.5.min.js"></script>
</head>

<!-- paulirish.com/2008/conditional-stylesheets-vs-css-hacks-answer-neither/ -->

<!--[if lt IE 7 ]> <body class="ie6"> <![endif]-->
<!--[if IE 7 ]>    <body class="ie7"> <![endif]-->
<!--[if IE 8 ]>    <body class="ie8"> <![endif]-->
<!--[if IE 9 ]>    <body class="ie9"> <![endif]-->
<!--[if (gt IE 9)|!(IE)]><!--> <body> <!--<![endif]-->

  <div id="global-title-container">
    <div id="title-wrapper" class="clearfix">
      <div id="title-left">
        <a href="http://www.gersteinlab.org"><strong>Gerstein</strong> Lab</a>
      </div>
      <div id="title-right">
        <a href="http://cbb.yale.edu">Bioinformatics</a>
      </div>
    </div>
  </div>
  <div id="global-container">
    <div id="global-main" class="clearfix">
      <div id="global-full">
        <h1>Homepage image upload</h1>
        
        <? $upload_success = FALSE; ?>
        <? if ($_POST): ?>
          <? $upload_success = handle_upload(); ?>
        <? endif; ?>
        <? if ($upload_success): ?>
          <p>File upload success!</p>
          <p>
            <? echo $blurb; ?>
          </p>
          <p>
            <a href="index.php">Upload another file</a>
          </p>
        <? else: ?>
          
          <? if ( ! empty($fatal)): ?>
          <ul>
          <? foreach ($fatal as $fatal_msg): ?>
            <li><span style="color: red;"><? echo $fatal_msg; ?></span></li>
          <? endforeach; ?>
          </ul>
          <? endif; ?>
          <h2>Upload form info</h2>
          <p>
            The file name of the image file must be in the following format: paperid__str.ext (note: this is a double underscore) where 'paperid' is a valid paper ID and 'str' may be any arbitrary string and 'ext' is a valid extension for an image file. The image must be under 500 MB and have dimensions <? echo IMG_WIDTH; ?> width x <?echo IMG_HEIGHT; ?> height. Please see <a href="http://wiki.gersteinlab.org/labinfo/Homepage_image_gallery" target="_blank">the wiki page</a> for more detailed documentation. 
          </p>
          <p>
            For all currently uploaded images, see the <a href="./images/">image gallery</a>.
          </p>
          
          <form class="form-table" name="form1" enctype="multipart/form-data" method="POST" action="index.php">
          <table>
            <tr>
              <td class="form-label"><label for="uploadFile">File to Upload:</label></td>
              <td class="form-field">
                <input name="uploadFile" type="file" id="uploadFile">
              </td>
              <td class="form-field">
                <? print_error('uploadFile'); ?>
              </td>
            </tr>
            <tr>
              <td class="form-label"><label for="fileSubgroup">Subgroup:</label></td>
              <td class="form-field"> 
                <input name="fileSubgroup" type="text" id="fileSubgroup">
              </td>
              <td class="form-field">
                <? print_error('fileSubgroup'); ?>
              </td>
            </tr>
            <tr>
              <td class="form-label"><label for="fileSubmitter">Submitted by:</label></td>
              <td class="form-field">
                <input name="fileSubmitter" type="text" id="fileSubmitter" size="6">
              </td>
              <td class="form-field">
                <? print_error('fileSubmitter'); ?>
              </td>
            </tr>
            <tr>
              <td class="form-label"><label for="fileDescription">Description:</label></td>
              <td class="form-field">
                <input name="fileDescription" type="textarea" id="fileDescription" size="40">
              </td>
              <td class="form-field">
                <? print_error('fileDescription'); ?>
              </td>
            </tr>
            <tr>
              <td class="form-submit" colspan="3">
                <input type="submit" name="Submit" value="Submit">
              </td>
            </tr>
          </table>
          </form>
        <? endif; ?>
          <h2>Rebuild</h2>
        
        <? if (isset($_GET['rebuild'])): ?>
        <? //if (false): ?>
          <? if (($rebuild_success = rebuild_homepage()) == TRUE): ?>
          <p><span style="color:green;">Homepage successfully rebuilt</span></p>
          <? else: ?>
          <p><span style="color:red;">Error rebuilding homepage</span></p>
          <? endif; ?>
        <? //endif; ?>
        <? endif; ?>
        
          <p>
Step #1: click rebuild homepage below.<br>
            <a href="index.php?rebuild=1">Rebuild homepage</a>
            <br>	
Step #2: after the rebuild has successfully completed click rebuild rotator below.<br>
	    <a href="http://www.gersteinlab.org/media/rotator-rebuild/">Rebuild rotator</a>
          </p>
      </div>
    </div>
    <div id="global-footer">
      &copy; <a href="http://info.gersteinlab.org/Permissions">2011</a> <strong><a href="http://www.yale.edu">Yale University</a></strong>
    </div>
  </div>
</body>
</html>
