#!/usr/bin/python 

# This script runs from gondor2

import sys, os 
import shutil
from datetime import datetime
import logging, logging.handlers
from urllib2 import Request, urlopen, URLError, HTTPError

import socket, atom.service, gdata.service
import gdata.spreadsheet, gdata.spreadsheet.service
import gdata.spreadsheet.text_db

gd_client = gdata.spreadsheet.service.SpreadsheetsService()
gd_client.email = 'programmer@gersteinlab.org'
gd_client.password = 'alpaca'
try:                    
   gd_client.ProgrammaticLogin()
except socket.sslerror, e:
   logging.error('Spreadsheet socket.sslerror: ' + str(e))

papers_master='0AiiHlTECOi8edGhzSXlZZzEyRThQeDB6R0pRc0FvcGc'
papers_xml_import='0AiiHlTECOi8edHFxQWRwN0kxeTQ3ZzdCOXhQX2Z1ZGc'
worksheet_id='od6'

try:
   feed_master = gd_client.GetListFeed (papers_master, worksheet_id ).entry
   feed_xml = gd_client.GetListFeed (papers_xml_import, worksheet_id ).entry
except gdata.service.RequestError, e:
   logging.error('Spreadsheet gdata.service.RequestError: ' + str(e))
except socket.sslerror, e:
   logging.error('Spreadsheet socket.sslerror: ' + str(e))

IMAGE_DIR     = './images'
JS_OUT        = './js/rotator-mf.js'
JS_SEQ_OUT    = './js/rotator_seq-mf.js'
## uncomment during production
RSYNC_IMAGES  = "rsync -rlvz -e 'ssh -i /nfs/home/common/sec/id_rsa.002.dzc2' ./images/ dzc2@www2.gersteinlab.org:/var/www/www/media/images/"
RSYNC_JS      = "rsync -rlvz -e 'ssh -i /nfs/home/common/sec/id_rsa.002.dzc2' ./js/ dzc2@www2.gersteinlab.org:/var/www/www/media/js"
LOGFILE       = 'synclog-mf.txt'
LOG_TO_FILE   = True
ACCEPTED_EXTS = ['.jpg', '.jpeg', '.png', '.gif']

# Globals
logger        = None

# Function: log_msg(src, msg)
# 
# Logs message in format [datetime]: message
# @param file
# @param msg

def log_msg(file, msg):
    global logger
    
    if LOG_TO_FILE == True:
        if file == sys.stderr:
            logger.error(msg)
        else:
            logger.debug(msg)
    else:
        print >> file, "[%s]: %s" % (str(datetime.datetime.now()), msg)

# Function: build_url(filename)
# 
# Build a URL using the name portion of the filename as paper id
# @param filename

def build_url(filename):
    stripped_filename = os.path.splitext(filename)[0]
    pos = stripped_filename.find('__')
    paper_id = stripped_filename[:pos]
    return 'http://papers.gersteinlab.org/papers/%s/' % paper_id

# Function: get_title(filename)
# 
# Grab the paper description using the name portion of the filename as paper id
# @param filename 

def get_title(filename):
    stripped_filename = os.path.splitext(filename)[0]
    pos = stripped_filename.find('__')
    paper_id = stripped_filename[:pos]


    pmid = '0'
    paper_title=''
    for mrow in feed_master:
       if paper_id == mrow.custom['labid'].text:
#          print "labid> %s:%s" % (paper_id, mrow.custom['labid'].text)
          pmid = mrow.custom['pmid'].text
          for xrow in feed_xml:
 #            print "pmid>"
             if pmid == xrow.custom['pmid'].text:
                paper_title = xrow.custom['title'].text + " " + xrow.custom['journal'].text + " (" + xrow.custom['year'].text + ")"
             
#       print " labid: %s \t title: %s" % (row.custom['labid'].text, row.custom['title'].text)
#	for key in row.custom:
#		print " %s: %s" % (key, row.custom[key].text)
    return paper_title

# Function: write_js(files)
#
# Takes a list of filenames and URL's generated from the filenames and writes the JavaScript
# used to rotate those images and links

def write_js(files):
    global JS_OUT
    
    try:
        out = open(JS_OUT, 'w')
    except:
        log_msg(sys.stderr, 'Could not open js file %s' % JS_OUT)
        raise

    out.write("function rotateImages()\n")
    out.write("{\n")

    out.write("\tvar a = new Array(\n")
    i = 0
    for filename, url in files:
        if i < len(files) - 1: 
            out.write("\t\t\"/media/images/" + filename + "\",\n")
        else:
            out.write("\t\t\"/media/images/" + filename + "\"\n")
        i = i + 1
    out.write("\t);\n")

    out.write("\tvar c = new Array(\n")
    i = 0
    for filename, url in files:
        if i < len(files) - 1:
            out.write("\t\t\"" + url + "\",\n")
        else:
            out.write("\t\t\"" + url + "\"\n")
        i = i + 1
    out.write("\t);\n")

# Added by MF
    out.write("\tvar t = new Array(\n")
    i = 0
    for filename, url in files:
        print 'I am here in the first for loop\n'
        paper_description = get_title(filename)
        if i < len(files) - 1:
            out.write("\t\t\"" + paper_description + "\",\n")
        else:
            out.write("\t\t\"" + paper_description + "\"\n")
        i = i + 1
    out.write("\t);\n")
# End added by MF

    out.write("\tvar b = document.getElementById(\"rotator-anchor\");\n")
    out.write("\tvar d = document.getElementById(\"rotator-image\");\n")
    out.write("\tvar i = Math.floor(Math.random() * a.length);\n")
    out.write("\n")
    out.write("\td.src  = a[i];\n")
    out.write("\tb.href = c[i];\n")

# Added by MF
    out.write("\tb.title = t[i];\n")
# End added by MF

    out.write("\n")
    out.write("\twindow.setTimeout(\"rotateImages()\", 6000);\n")
    out.write("}\n")
    out.write("\n")
    out.write("rotateImages();\n")

    out.close()

# Function: write_js_seq(files)
#
# Takes a list of filenames and URL's generated from the filenames and writes the JavaScript
# used to rotate those images and links

def write_js_seq(files):
    global JS_SEQ_OUT
    
    try:
        out = open(JS_SEQ_OUT, 'w')
    except:
        log_msg(sys.stderr, 'Could not open js file %s' % JS_SEQ_OUT)
        raise

    out.write("function rotateImages(start)\n")
    out.write("{\n")
    out.write("\tvar a = new Array(\n")
    i = 0
    for filename, url in files:
        if i < len(files) - 1: 
            out.write("\t\t\"/media/images/" + filename + "\",\n")
        else:
            out.write("\t\t\"/media/images/" + filename + "\"\n")
        i = i + 1
    out.write("\t);\n")
    out.write("\tvar c = new Array(\n")
    i = 0
    for filename, url in files:
        if i < len(files) - 1:
            out.write("\t\t\"" + url + "\",\n")
        else:
            out.write("\t\t\"" + url + "\"\n")
        i = i + 1
    out.write("\t);\n")

# Added by MF
    out.write("\tvar t = new Array(\n")
    i = 0
    for filename, url in files:
        print 'I am here in the second for loop\n'
        paper_description = get_title(filename)
        if i < len(files) - 1:
            out.write("\t\t\"" + paper_description + ",\n")
        else:
            out.write("\t\t\"" + paper_description + "\"\n")
        i = i + 1
    out.write("\t);\n")
# End added by MF
    out.write("\tvar b = document.getElementById(\"rotator-anchor\");\n")
    out.write("\tvar d = document.getElementById(\"rotator-image\");\n")
    out.write("\tif (start >= a.length)\n")
    out.write("\t\tstart = 0;\n")
    out.write("\n")
    out.write("\td.src  = a[start];\n")
    out.write("\tb.href = c[start];\n")
# Added by MF
    out.write("\tb.title = t[start];\n")
# End added by MF
    out.write("\n")
    out.write("\twindow.setTimeout(\"rotateImages(\" + (start + 1) + \")\", 3000);\n")
    out.write("}\n")
    out.write("\n")
    out.write("rotateImages(0);\n")

    out.close()
      
# Main method
def main(argv=None):
    global IMAGE_DIR, logger
    
    if argv is None:
        argv = sys.argv
        
    # Initiate logging
    logger = logging.getLogger("syncsplash")
    log_handler = logging.FileHandler("syncsplash.log")
    log_formatter = logging.Formatter('[%(asctime)s] %(levelname)s %(message)s')
    
    log_handler.setFormatter(log_formatter)
    logger.addHandler(log_handler)
    logger.setLevel(logging.DEBUG)

    # List of source files and out files
    src_files = []
    out_files = []
    
    try:
        src_files = os.listdir(IMAGE_DIR)
    except:
        log_msg(sys.stderr, 'Cannot read images directory')
        return 1
    
    for filename in src_files:
        extension = os.path.splitext(filename)[1]
        if extension not in ACCEPTED_EXTS:
            continue
        
        paper_url = build_url(filename)
        out_files.append([filename, paper_url])
        
    # Write JavaScript
    try:
        write_js(out_files)
        write_js_seq(out_files)
    except:
        log_msg(sys.stderr, 'Could not write js')
        return 3
    
    os.putenv('RSYNC_PASSWORD', 'Madrigal24')
    
    
    # Invoke rsync
    ret = os.system(RSYNC_IMAGES)
    if ret != 0:
        log_msg(sys.stderr, 'rsyncing image failed: %d' % ret)
        return 2
    
    ret = os.system(RSYNC_JS)
    if ret != 0:
        log_msg(sys.stderr, 'rsyncing javascript failed: %d' % ret)
        return 2
        
    return 0

if __name__ == "__main__":
    sys.exit(main())
