#!/usr/bin/python

import sys
import os
import datetime
import shutil
import logging, logging.handlers
#from PIL import Image
from urllib2 import Request, urlopen, URLError, HTTPError

# this should work from homes.gersteinlab but not from gondor
IMAGE_DIR     = './images'
JS_OUT        = './js/rotator-mf.js'
JS_SEQ_OUT    = './js/rotator_seq-mf.js'
# RSYNC_IMAGES  = "rsync -rlvz -e 'ssh -i /var/www/.ssh/id_rsa.002' /var/www/html/images/images/ dzc2@www.gersteinlab.org:/var/www/www/media/images/"
# RSYNC_JS      = "rsync -rlvz -e 'ssh -i /var/www/.ssh/id_rsa.002' /var/www/html/images/js/ dzc2@www.gersteinlab.org:/var/www/www/media/js"
RSYNC_IMAGES  = "rsync -rlvz -e 'ssh -i /var/www/.ssh/gersteinlab_rsa' /var/www/html/images/images/ glrotator@www.gersteinlab.org:/var/www/www/media/images/"
RSYNC_JS      = "rsync -rlvz -e 'ssh -i /var/www/.ssh/gersteinlab_rsa' /var/www/html/images/js/ glrotator@www.gersteinlab.org:/var/www/www/media/js"

## FIXPERMS      = "ssh -i /var/www/.ssh/id_rsa.002 dzc2@www.gersteinlab.org chmod 664 /var/www/www/media/images/*.*g*"
LOGFILE       = 'synclog.txt'
LOG_TO_FILE   = True
ACCEPTED_EXTS = ['.jpg', '.jpeg', '.png', '.gif']

# Globals
logger        = None

# Function: log_msg(src, msg)
# 
# Logs message in format [datetime]: message
# @param file
# @param msg

def log_msg(file, msg):
    global logger
    
    if LOG_TO_FILE == True:
        if file == sys.stderr:
            logger.error(msg)
        else:
            logger.debug(msg)
    else:
        print >> file, "[%s]: %s" % (str(datetime.datetime.now()), msg)
        


# Function: build_url(filename)
# 
# Build a URL using the name portion of the filename as paper id
# @param filename

def build_url(filename):
    stripped_filename = os.path.splitext(filename)[0]
    pos = stripped_filename.find('__')
    paper_id = stripped_filename[:pos]
    return 'http://papers.gersteinlab.org/papers/%s/' % paper_id



# Function: write_js(files)
#
# Takes a list of filenames and URL's generated from the filenames and writes the JavaScript
# used to rotate those images and links

def write_js(files):
    global JS_OUT
    
    try:
        out = open(JS_OUT, 'w')
    except:
        log_msg(sys.stderr, 'Could not open js file %s' % JS_OUT)
        raise

    out.write("function rotateImages()\n")
    out.write("{\n")
    out.write("\tvar a = new Array(\n")
    i = 0
    for filename, url in files:
        if i < len(files) - 1: 
            out.write("\t\t\"/media/images/" + filename + "\",\n")
        else:
            out.write("\t\t\"/media/images/" + filename + "\"\n")
        i = i + 1
    out.write("\t);\n")
    out.write("\tvar c = new Array(\n")
    i = 0
    for filename, url in files:
        if i < len(files) - 1:
            out.write("\t\t\"" + url + "\",\n")
        else:
            out.write("\t\t\"" + url + "\"\n")
        i = i + 1
    out.write("\t);\n")

    out.write("\tvar b = document.getElementById(\"rotator-anchor\");\n")
    out.write("\tvar d = document.getElementById(\"rotator-image\");\n")
    out.write("\tvar i = Math.floor(Math.random() * a.length);\n")
    out.write("\n")
    out.write("\td.src  = a[i];\n")
    out.write("\tb.href = c[i];\n")
    out.write("\n")
    out.write("\twindow.setTimeout(\"rotateImages()\", 4000);\n")
    out.write("}\n")
    out.write("\n")
    out.write("rotateImages();\n")

    out.close()

# Function: write_js_seq(files)
#
# Takes a list of filenames and URL's generated from the filenames and writes the JavaScript
# used to rotate those images and links

def write_js_seq(files):
    global JS_SEQ_OUT
    
    try:
        out = open(JS_SEQ_OUT, 'w')
    except:
        log_msg(sys.stderr, 'Could not open js file %s' % JS_SEQ_OUT)
        raise

    out.write("function rotateImages(start)\n")
    out.write("{\n")
    out.write("\tvar a = new Array(\n")
    i = 0
    for filename, url in files:
        if i < len(files) - 1: 
            out.write("\t\t\"/media/images/" + filename + "\",\n")
        else:
            out.write("\t\t\"/media/images/" + filename + "\"\n")
        i = i + 1
    out.write("\t);\n")
    out.write("\tvar c = new Array(\n")
    i = 0
    for filename, url in files:
        if i < len(files) - 1:
            out.write("\t\t\"" + url + "\",\n")
        else:
            out.write("\t\t\"" + url + "\"\n")
        i = i + 1
    out.write("\t);\n")

    out.write("\tvar b = document.getElementById(\"rotator-anchor\");\n")
    out.write("\tvar d = document.getElementById(\"rotator-image\");\n")
    out.write("\tif (start >= a.length)\n")
    out.write("\t\tstart = 0;\n")
    out.write("\n")
    out.write("\td.src  = a[start];\n")
    out.write("\tb.href = c[start];\n")
    out.write("\n")
    out.write("\twindow.setTimeout(\"rotateImages(\" + (start + 1) + \")\", 3000);\n")
    out.write("}\n")
    out.write("\n")
    out.write("rotateImages(0);\n")

    out.close()
      

# Main method
def main(argv=None):
    global IMAGE_DIR, logger
    
    if argv is None:
        argv = sys.argv
        
    # Initiate logging
    logger = logging.getLogger("syncsplash")
    log_handler = logging.FileHandler("syncsplash.log")
    log_formatter = logging.Formatter('[%(asctime)s] %(levelname)s %(message)s')
    
    log_handler.setFormatter(log_formatter)
    logger.addHandler(log_handler)
    logger.setLevel(logging.DEBUG)

    # List of source files and out files
    src_files = []
    out_files = []
    
    try:
        src_files = os.listdir(IMAGE_DIR)
    except:
        log_msg(sys.stderr, 'Cannot read images directory')
        return 1
    
    for filename in src_files:
        extension = os.path.splitext(filename)[1]
        if extension not in ACCEPTED_EXTS:
            continue
        
        paper_url = build_url(filename)
        out_files.append([filename, paper_url])
        
    # Write JavaScript
    try:
        write_js(out_files)
        write_js_seq(out_files)
    except:
        log_msg(sys.stderr, 'Could not write js')
        return 3
    
    os.putenv('RSYNC_PASSWORD', 'Madrigal24')
    
    
    # Invoke rsync
    ret = os.system(RSYNC_IMAGES)
    if ret != 0:
        log_msg(sys.stderr, 'rsyncing image failed: %d' % ret)
        return 2
    
    ret = os.system(RSYNC_JS)
    if ret != 0:
        log_msg(sys.stderr, 'rsyncing javascript failed: %d' % ret)
        return 2

# MF attempt to fix permissions on www
#    ret = os.system(FIXPERMS)
#    if ret != 0:
#        log_msg(sys.stderr, 'permission change failed: %d' % ret)
#        return 2

    return 0

if __name__ == "__main__":
    sys.exit(main())
